require 'dsx/dml/version'

module Dsx
  module Dml
    
    class Zone

      def initialize(location_num, udf_field_num, writer_options=false)
        @location_num = location_num
        @udf_field_num = udf_field_num
        if(writer_options)
          @writer = Dsx::Dml::Writer.new(writer_options)
        end
      end
      
      def use(id_num)
        if(@writer)
          return Changeset.new(self, id_num)
        else
          return Changeset.new(self, id_num, @writer)
        end
      end
      
      def location_num
        @location_num
      end
      
      def udf_field_num
        @udf_field_num
      end
      
    end
    
    
    class Changeset
    
      def initialize(zone, id_num, writer = false)
        @id_num = id_num
        @zone = zone
        @stack = []
        if(writer)
          @writer = writer
        end
        start_command
      end
    
      def table(table_name)
        case table_name
        when :Cards
            return CardsTable.new(self, table_name)
        when :Names
            return NamesTable.new(self, table_name)
        when 'Cards'
            return CardsTable.new(self, table_name)
        when 'Names'
            return NamesTable.new(self, table_name)
        else
            return Table.new(self, table_name)
        end
      end
        
      # Their system won't work correctly with \n, but will with \r. 
      # It'd also be possible to change the record separator, but that seems like a worse idea.
      def to_string
        @stack.join("\r")
      end
      
      def to_s
        to_string
      end
      
      def writer=(writer)
        @writer = writer
      end
        
      def writer
        @writer
      end
    
      def net_sftp(writer = false)
        if (writer)
            writer.net_sftp(to_string)
        elsif (@writer)
            @writer.net_sftp(to_string)
        end
        return self
      end
      
      # Alternative to Net::SFTP.
      def curl(writer = false)
        if (writer)
            writer.curl(to_string)
        elsif (@writer)
            @writer.curl(to_string)
        end
        return self
      end
        
      def stack
        @stack
      end
        
      private
        
      def start_command
        @stack << "I L#{@zone.location_num} U#{@zone.udf_field_num} ^#{@id_num}^^^"
      end
      
    end
    
    
    class Table
    
      def initialize(changeset, table_name)
        @changeset = changeset
        @table_name = table_name
        @changeset.stack << "T #{@table_name}"
      end
        
      def table(table_name)
        @changeset.table(table_name)
      end
        
      def done
        @changeset
      end
        
      def batch
        @changeset
      end
      
      def net_sftp(params)
        @changeset.net_sftp(params)
      end
      
      def curl(params)
        @changeset.curl(params)
      end
      
      def queue(data)
        fields(data)
      end
        
      def write(data={})
        fields(data)
        @changeset.stack << 'W'
        return self
      end
        
      def update(data={})
        fields(data)
        @changeset.stack << 'U'
        return self
      end
        
      def delete
        fields(data={})
        @changeset.stack << 'D'
        return self
      end
      
      private
      
      def fields(hash)
        hash.each { |k,v| single(k,v) }
      end
      
      def blank(command)
        single(command, '')
      end
      
      def single(command, val)
        @changeset.stack << "F #{command} ^#{val}^^^"
        return self
      end
        
    end
    
    
    class CardsTable < Table
      
      def relace_code(code)
        single('ReplaceCode', code)
      end
  
      def copy_to_code(code)
        single('Copy2Code', code)
      end
  
      def add_acl(acl)
        single('AddAcl', acl)
      end
  
      def add_tmp_acl(acl)
        single('AddTempAcl', acl)
      end
  
      def del_acl(acl)
        single('DelAcl', acl)
      end
  
      def del_tmp_acl(acl)
        single('DelTempAcl', acl)
      end
  
      def clear_all_acl
        blank('ClearAcl')
      end
  
      def clear_all_tmp_acl
        blank('ClearTempAcl')
      end
  
      def copy_perm_to_tmp
        blank('CopyPerm2Temp')
      end
  
    end
  

    class NamesTable < Table
      
      def dc
        @changeset.stack << 'DC'
      end
      
    end
    
  end
end
