require 'dsx/dml/version'
require 'dsx/dml'
require 'dsx/writer'



module Dsx
  module Dml
  
    class Operation
    
        def initialize(details, writer_options = false)
          initialize_details(details)
          initialize_command(writer_options)
          
        end
        
        def last_type=(type)
          @type = type
        end
          
        def last_type
          @type
        end
        
        def reset_cards
          table = @changeset.table(:Names)
          table.dc
          table.write
          self
        end
        
        def access_levels
          return AccessLevelOp.new(self, @changeset)
        end
        
        def access_levels=(args=false)
          return self
        end
        
        def linking_level=(level)
          table = @changeset.table(:Cards)
          table.write({Code: @identifier, Loc: @location_num, OLL: level})
          self
        end
        
        def tmp_access_levels
          return TmpAccessLevelOp.new(self, @changeset)
        end
        
        def identifier
          @identifier
        end
        
        def changeset
          @changeset
        end
        
        def to_s
          @changeset.to_s
        end
        
        def net_sftp(writer=false)
          @changeset.net_sftp(writer)
        end
        
        def curl(writer=false)
          @changeset.curl(writer)
        end
        
        private
        
        def initialize_command(writer_options=false)
          @zone = (writer_options) ? Zone.new(@location_num, @udf_num, writer_options) : Zone.new(@location_num, @udf_num)
          changeset = @zone.use(@identifier)
          changeset.table(:Names).write(FName: @first_name, 
                                         LName: @last_name, 
                                         Company: @company)
          changeset.table(:UDF).write(UdfNum: @udf_num,
                                       UdfText: @identifier)
          @changeset = changeset
        end
        
        def initialize_details(details)
           if( details.key?(:location_num) &&
               details.key?(:udf_num) &&
               details.key?(:first_name) &&
               details.key?(:last_name) &&
               details.key?(:company)&&
               details.key?(:identifier) )
            @location_num = details[:location_num]
            @udf_num = details[:udf_num]
            @first_name = details[:first_name]
            @last_name = details[:last_name]
            @company = details[:company]
            @identifier = details[:identifier]
          else
            raise "Operation arguments missing"
          end
        end
    end
    
    
    class OpTemplate
      def initialize(op, changeset)
        @changeset = changeset
        @op = op
      end
    end
    
    
    class CardRelatedOp < OpTemplate
      def initialize(op, changeset)
        super(op, changeset)
        @table = @changeset.table(:Cards)
        @table.queue(Code: op.identifier)
      end
    end
    
    
    class AccessLevelOp < CardRelatedOp
      
      def + (values)
        values.each do |level|
          @table.add_acl(level)
        end
        @table.write
        @op
      end
      
      def - (values)
        values.each do |level|
          @table.del_acl(level)
        end
        @table.write
        @op
      end
      
      def reset
        @table.clear_all_acl
        @table.write
        @op
      end
    
    end
    
    
    class TmpAccessLevelOp < CardRelatedOp
    
      def + (values)
        values.each do |level|
          @table.add_tmp_acl(level)
        end
        @table.write
        @op
      end
      
      def - (values)
        values.each do |level|
          @table.del_tmp_acl(level)
        end
        @table.write
        @op
      end
      
      def reset
        @table.clear_all_tmp_acl
        @table.write
        @op
      end
    
    end

    
    
  end
end
    
